Improve security checks



git-svn-id: https://svn.apache.org/repos/asf/felix/trunk@912098 13f79535-47bb-0310-9956-ffa450edef68
diff --git a/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/instantiator/ServiceImporter.java b/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/instantiator/ServiceImporter.java
index 6036f1c..ed63da7 100644
--- a/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/instantiator/ServiceImporter.java
+++ b/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/instantiator/ServiceImporter.java
@@ -74,7 +74,8 @@
                 m_reg.unregister();

             }

             m_svcObject = getService(m_ref);

-            m_reg = m_handler.getCompositeManager().getServiceContext().registerService(getSpecification().getName(), m_svcObject, getProps(m_ref));

+            m_reg = m_handler.getCompositeManager().getServiceContext()

+                .registerService(getSpecification().getName(), m_svcObject, getProps(m_ref));

         }

 

         /**

diff --git a/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/provides/ServiceExporter.java b/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/provides/ServiceExporter.java
index 906a2d2..5ee663a 100644
--- a/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/provides/ServiceExporter.java
+++ b/ipojo/composite/src/main/java/org/apache/felix/ipojo/composite/service/provides/ServiceExporter.java
@@ -30,6 +30,7 @@
 import org.apache.felix.ipojo.composite.CompositeManager;

 import org.apache.felix.ipojo.util.DependencyModel;

 import org.apache.felix.ipojo.util.DependencyStateListener;

+import org.apache.felix.ipojo.util.SecurityHelper;

 import org.osgi.framework.BundleContext;

 import org.osgi.framework.Filter;

 import org.osgi.framework.ServiceReference;

@@ -127,8 +128,15 @@
      */

     public void onServiceArrival(ServiceReference reference) {

         Object svc = getService(reference);

-        ServiceRegistration reg = m_destination.registerService(getSpecification().getName(), svc, getProps(reference));

-        m_registrations.put(reference, reg);

+        // Security Check

+        if (SecurityHelper.hasPermissionToRegisterService(getSpecification().getName(), m_destination)) {

+            ServiceRegistration reg = m_destination

+                .registerService(getSpecification().getName(), svc, getProps(reference));

+            m_registrations.put(reference, reg);

+        } else {

+            throw new SecurityException("The bundle " + m_destination.getBundle().getBundleId() + " does not have the " +

+                    "permission to register the service " + getSpecification().getName());

+        }

     }

 

     /**

diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/EventDispatcher.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/EventDispatcher.java
index f867942..8219422 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/EventDispatcher.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/EventDispatcher.java
@@ -52,6 +52,7 @@
     private Map m_listeners;
     /**
      * The global bundle context.
+     * This is the bundle context from iPOJO.
      */
     private BundleContext m_context;
     
diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/Extender.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/Extender.java
index 14345b7..40a7c86 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/Extender.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/Extender.java
@@ -82,7 +82,7 @@
     /**

      * The Bundle Context of the iPOJO Core bundle.

      */

-    private BundleContext m_context;

+    private static BundleContext m_context;

 

     /**

      * The instance creator used to create instances.

@@ -342,6 +342,15 @@
         m_creator = null;

         

         m_logger.log(Logger.INFO, "iPOJO Runtime stopped");

+        m_context = null;

+    }

+    

+    /**

+     * Gets iPOJO bundle context.

+     * @return the iPOJO Bundle Context

+     */

+    public static BundleContext getIPOJOBundleContext() {

+        return m_context;

     }

     

     /**

diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/IPojoFactory.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/IPojoFactory.java
index 6c4956c..e0454de 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/IPojoFactory.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/IPojoFactory.java
@@ -31,6 +31,7 @@
 import org.apache.felix.ipojo.architecture.PropertyDescription;

 import org.apache.felix.ipojo.metadata.Element;

 import org.apache.felix.ipojo.util.Logger;

+import org.apache.felix.ipojo.util.SecurityHelper;

 import org.osgi.framework.BundleContext;

 import org.osgi.framework.Constants;

 import org.osgi.framework.ServiceReference;

@@ -585,14 +586,24 @@
 

         if (m_isPublic) {

             // Exposition of the factory service

+            BundleContext bc = SecurityHelper.selectContextToRegisterServices(m_componentDesc.getFactoryInterfacesToPublish(), 

+                    m_context, getIPOJOBundleContext());

             m_sr =

-                    m_context.registerService(m_componentDesc.getFactoryInterfacesToPublish(), this, m_componentDesc

+                    bc.registerService(m_componentDesc.getFactoryInterfacesToPublish(), this, m_componentDesc

                             .getPropertiesToPublish());

         }

         

         m_logger.log(Logger.INFO, "Factory " + m_factoryName + " started");

 

     }

+    

+    /**

+     * Gets the iPOJO Bundle Context.

+     * @return the iPOJO Bundle Context

+     */

+    protected final BundleContext getIPOJOBundleContext() {

+        return Extender.getIPOJOBundleContext();

+    }

 

     /**

      * Creates or updates an instance.

diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/configuration/ConfigurationHandler.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/configuration/ConfigurationHandler.java
index abd6629..9cecf0d 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/configuration/ConfigurationHandler.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/configuration/ConfigurationHandler.java
@@ -38,6 +38,7 @@
 import org.apache.felix.ipojo.parser.PojoMetadata;
 import org.apache.felix.ipojo.util.Callback;
 import org.apache.felix.ipojo.util.Property;
+import org.apache.felix.ipojo.util.SecurityHelper;
 import org.osgi.framework.Constants;
 import org.osgi.framework.ServiceRegistration;
 import org.osgi.service.cm.ManagedService;
@@ -294,7 +295,16 @@
             props.put(Constants.SERVICE_PID, m_managedServicePID);
             props.put("instance.name", getInstanceManager().getInstanceName());
             props.put("factory.name", getInstanceManager().getFactory().getFactoryName());
-            m_sr = getInstanceManager().getContext().registerService(ManagedService.class.getName(), this, props);
+            
+            // Security Check
+            if (SecurityHelper.hasPermissionToRegisterService(ManagedService.class.getName(), 
+                    getInstanceManager().getContext())) {
+                m_sr = getInstanceManager().getContext().registerService(ManagedService.class.getName(), this, props);
+            } else {
+                error("Cannot register the ManagedService - The bundle " 
+                        + getInstanceManager().getContext().getBundle().getBundleId() 
+                        + " does not have the permission to register the service");
+            }
         }
     }
 
diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Logger.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Logger.java
index 2cb6728..3c8029c 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Logger.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Logger.java
@@ -18,6 +18,7 @@
  */
 package org.apache.felix.ipojo.util;
 
+import org.apache.felix.ipojo.Extender;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
 import org.osgi.service.log.LogService;
@@ -132,7 +133,13 @@
         LogService log = null;
         ServiceReference ref = null;
         try {
-            ref = m_context.getServiceReference(LogService.class.getName());
+            // Security Check
+            if (SecurityHelper.hasPermissionToGetService(LogService.class.getName(), m_context)) {
+                ref = m_context.getServiceReference(LogService.class.getName());
+            } else {
+                Extender.getIPOJOBundleContext().getServiceReference(LogService.class.getName());
+            }
+            
             if (ref != null) {
                 log = (LogService) m_context.getService(ref);
             }
@@ -195,7 +202,13 @@
         LogService log = null;
         ServiceReference ref = null;
         try {
-            ref = m_context.getServiceReference(LogService.class.getName());
+            // Security Check
+            if (SecurityHelper.hasPermissionToGetService(LogService.class.getName(), m_context)) {
+                ref = m_context.getServiceReference(LogService.class.getName());
+            } else {
+                Extender.getIPOJOBundleContext().getServiceReference(LogService.class.getName());
+            }
+            
             if (ref != null) {
                 log = (LogService) m_context.getService(ref);
             }
diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/util/SecurityHelper.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/SecurityHelper.java
new file mode 100644
index 0000000..ce8d210
--- /dev/null
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/SecurityHelper.java
@@ -0,0 +1,186 @@
+/* 
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+package org.apache.felix.ipojo.util;
+
+import java.security.Permission;
+
+import org.osgi.framework.BundleContext;
+import org.osgi.framework.ServicePermission;
+
+
+/**
+ * Methods checking security permissions
+ * @author <a href="mailto:dev@felix.apache.org">Felix Project Team</a>
+ */
+public class SecurityHelper {
+    
+    
+    /**
+     * Gets a bundle context to register the given services.
+     * This method can be used only if iPOJO is able to
+     * registers the services (so for ManagedServiceFactory,
+     * Factory and Architecture)
+     * @param itf the service interfaces
+     * @param comp the component bundle context
+     * @param ipojo the ipojo bundle context
+     * @return <code>comp</code> if the bundle has enough permission
+     * to register the service, <code>ipojo</code> otherwise.
+     */
+    public static BundleContext selectContextToRegisterServices(String[] itfs,
+            BundleContext comp, BundleContext ipojo) {
+        if (System.getSecurityManager() != null) {
+            for (int i = 0; i < itfs.length; i++) {
+                final Permission perm = new ServicePermission(itfs[i],
+                        ServicePermission.REGISTER);
+                if (!comp.getBundle().hasPermission(perm)) {
+                    return ipojo;
+                }
+            }
+            
+        }
+        return comp;
+    }
+    
+    /**
+     * Gets a bundle context to register the given service.
+     * This method can be used only if iPOJO is able to
+     * registers the service (so for ManagedServiceFactory,
+     * Factory and Architecture)
+     * @param itf the service interface
+     * @param comp the component bundle context
+     * @param ipojo the ipojo bundle context
+     * @return <code>comp</code> if the bundle has enough permission
+     * to register the service, <code>ipojo</code> otherwise.
+     */
+    public static BundleContext selectContextToRegisterService(String itf,
+            BundleContext comp, BundleContext ipojo) {
+        if (System.getSecurityManager() != null) {
+            final Permission perm = new ServicePermission(itf,
+                    ServicePermission.REGISTER);
+            if (!comp.getBundle().hasPermission(perm)) {
+                return ipojo;
+            }
+        }
+        return comp;
+    }
+    
+    /**
+     * Gets a bundle context to get the given service.
+     * This method can be used only if iPOJO is able to
+     * get the service (so for ManagedServiceFactory,
+     * Factory, Architecture and LogService)
+     * @param itf the service interface
+     * @param comp the component bundle context
+     * @param ipojo the ipojo bundle context
+     * @return <code>comp</code> if the bundle has enough permission
+     * to get the service, <code>ipojo</code> otherwise.
+     */
+    public static BundleContext selectContextToGetService(String itf,
+            BundleContext comp, BundleContext ipojo) {
+        if (System.getSecurityManager() != null) {
+            final Permission perm = new ServicePermission(itf,
+                    ServicePermission.GET);
+            if (!comp.getBundle().hasPermission(perm)) {
+                return ipojo;
+            }
+        }
+        return comp;
+    }
+    
+    /**
+     * Checks if the component bundle context has enough permission
+     * to get the given service.
+     * @param itf the service interface
+     * @param comp the component bundle context
+     * @return <code>true</code> if the bundle has enough permission
+     * to get the service, <code>false</code> otherwise.
+     */
+    public static boolean hasPermissionToGetService(String itf,
+            BundleContext comp) {
+        if (System.getSecurityManager() != null) {
+            final Permission perm = new ServicePermission(itf,
+                    ServicePermission.GET);
+            return comp.getBundle().hasPermission(perm);
+        }
+        return true;
+    } 
+    
+    /**
+     * Checks if the component bundle context has enough permission
+     * to get the given services.
+     * @param itf the service interfaces
+     * @param comp the component bundle context
+     * @return <code>true</code> if the bundle has enough permission
+     * to get the services, <code>false</code> otherwise.
+     */
+    public static boolean hasPermissionToGetServices(String[] itfs,
+            BundleContext comp) {
+        if (System.getSecurityManager() != null) {
+            for (int i = 0; i < itfs.length; i++) {
+                final Permission perm = new ServicePermission(itfs[i],
+                        ServicePermission.GET);
+                if (!comp.getBundle().hasPermission(perm)) {
+                    return false;
+                }
+            }
+        }
+        return true;
+    } 
+    
+    /**
+     * Checks if the component bundle context has enough permission
+     * to register the given service.
+     * @param itf the service interface
+     * @param comp the component bundle context
+     * @return <code>true</code> if the bundle has enough permission
+     * to register the service, <code>false</code> otherwise.
+     */
+    public static boolean hasPermissionToRegisterService(String itf,
+            BundleContext comp) {
+        if (System.getSecurityManager() != null) {
+            final Permission perm = new ServicePermission(itf,
+                    ServicePermission.REGISTER);
+            return comp.getBundle().hasPermission(perm);
+        }
+        return true;
+    }
+    
+    /**
+     * Checks if the component bundle context has enough permission
+     * to register the given services.
+     * @param itf the service interfaces
+     * @param comp the component bundle context
+     * @return <code>true</code> if the bundle has enough permission
+     * to register the services, <code>false</code> otherwise.
+     */
+    public static boolean hasPermissionToRegisterServices(String[] itfs,
+            BundleContext comp) {
+        if (System.getSecurityManager() != null) {
+            for (int i = 0; i < itfs.length; i++) {
+                final Permission perm = new ServicePermission(itfs[i],
+                        ServicePermission.REGISTER);
+                if (!comp.getBundle().hasPermission(perm)) {
+                    return false;
+                }
+            }
+        }
+        return true;
+    }
+
+}
diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Tracker.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Tracker.java
index 90b268d..0989734 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Tracker.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/util/Tracker.java
@@ -19,6 +19,7 @@
 package org.apache.felix.ipojo.util;

 

 import java.util.ArrayList;

+import java.util.Arrays;

 import java.util.HashMap;

 import java.util.Iterator;

 import java.util.LinkedList;

@@ -126,6 +127,12 @@
      *            the TrackerCustomizer object and the Tracker object will call the TrackerCustomizer methods on itself.    

      */

     public Tracker(BundleContext context, String clazz, TrackerCustomizer customizer) {

+        // Security Check

+        if (! SecurityHelper.hasPermissionToGetService(clazz, context)) {

+            throw new SecurityException("The bundle " + context.getBundle().getBundleId()

+                    + " does not have the permission to get the service " + clazz);

+        }

+        

         this.m_context = context;

         this.m_trackReference = null;

         this.m_trackClass = clazz;

@@ -414,6 +421,14 @@
      * @return the Service object or <code>null</code> if the service referenced by the specified ServiceReference object is not being tracked.

      */

     public Object getService(ServiceReference reference) {

+        // Security Check

+        if (! SecurityHelper.hasPermissionToGetServices((String[]) reference.getProperty(Constants.OBJECTCLASS), 

+                m_context)) {

+            throw new SecurityException("The bundle " + m_context.getBundle().getBundleId() + " does not have"

+                    + " the permission to get the services " 

+                    + Arrays.asList((String[]) reference.getProperty(Constants.OBJECTCLASS)));

+        }

+        

         Tracked tracked = this.m_tracked; // use local var since we are not synchronized

         if (tracked == null) { /* if Tracker is not open */

             return null;

diff --git a/ipojo/junit4osgi/maven-junit4osgi-plugin/pom.xml b/ipojo/junit4osgi/maven-junit4osgi-plugin/pom.xml
index 8072721..0386ea3 100644
--- a/ipojo/junit4osgi/maven-junit4osgi-plugin/pom.xml
+++ b/ipojo/junit4osgi/maven-junit4osgi-plugin/pom.xml
@@ -36,7 +36,7 @@
     <dependency>
       <groupId>org.apache.felix</groupId>
       <artifactId>org.apache.felix.framework</artifactId>
-      <version>2.0.2</version>
+      <version>2.0.4</version>
     </dependency>
     <dependency>
       <groupId>org.apache.maven</groupId>
diff --git a/ipojo/tests/core/logger/pom.xml b/ipojo/tests/core/logger/pom.xml
index 4a17d63..15e4b0d 100644
--- a/ipojo/tests/core/logger/pom.xml
+++ b/ipojo/tests/core/logger/pom.xml
@@ -67,7 +67,7 @@
   <dependency>
     <groupId>org.ops4j.pax.exam</groupId>
     <artifactId>pax-exam</artifactId>
-    <version>1.1.0</version>
+    <version>1.2.0</version>
   </dependency>
   <!--
     During runtime Pax Exam will discover the OSGi container to use by
@@ -79,7 +79,7 @@
     <groupId>org.ops4j.pax.exam</groupId>
     <artifactId>pax-exam-container-default
     </artifactId>
-    <version>1.1.0</version>
+    <version>1.2.0</version>
   </dependency>
   <!--
     If your test code is based on JUnit you will have to have the Junit
@@ -88,7 +88,7 @@
   <dependency>
     <groupId>org.ops4j.pax.exam</groupId>
     <artifactId>pax-exam-junit</artifactId>
-    <version>1.1.0</version>
+    <version>1.2.0</version>
   </dependency>
   <dependency>
     <groupId>junit</groupId>
@@ -101,7 +101,7 @@
   <dependency>
     <groupId>org.ops4j.pax.swissbox</groupId>
     <artifactId>pax-swissbox-tinybundles</artifactId>
-    <version>1.0.0</version>
+    <version>1.2.0</version>
   </dependency>
   <dependency>
     <groupId>org.apache.felix</groupId>
diff --git a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/BNDManifestLoggerInfoTest.java b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/BNDManifestLoggerInfoTest.java
index d194b53..9fcd1e4 100644
--- a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/BNDManifestLoggerInfoTest.java
+++ b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/BNDManifestLoggerInfoTest.java
@@ -7,7 +7,6 @@
 import static org.ops4j.pax.exam.CoreOptions.options;
 import static org.ops4j.pax.exam.CoreOptions.provision;
 import static org.ops4j.pax.exam.MavenUtils.asInProject;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.asURL;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.newBundle;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.withBnd;
 
@@ -83,23 +82,19 @@
                         ),
                 provision(
                         newBundle()
-                            .addClass( MyService.class )
-                            .prepare()
-                           .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
-                           .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                            .build( asURL() ).toExternalForm()
+                            .add( MyService.class )
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
+                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .build( withBnd() )
                     ),
                provision(
                        // Component
                         newBundle()
-                            .addClass(MyComponent.class)
-                            .prepare(
-                                    withBnd()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
-                                        .set(Constants.IMPORT_PACKAGE, "*")
-                                        .set("IPOJO-log-level", "info")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "provider-with-level-in-manifest.jar"), new File("component.xml"))).toExternalForm()
+                            .add(MyComponent.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
+                            .set(Constants.IMPORT_PACKAGE, "*")
+                            .set("IPOJO-log-level", "info")
+                            .build( asiPOJOBundle(new File(tmp, "provider-with-level-in-manifest.jar"), new File("component.xml")))
                             )
                 );
         return opt;
diff --git a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/ManifestLoggerInfoTest.java b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/ManifestLoggerInfoTest.java
index f9bf5ee..1eb453d 100644
--- a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/ManifestLoggerInfoTest.java
+++ b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/ManifestLoggerInfoTest.java
@@ -1,34 +1,37 @@
 package org.apache.felix.ipojo.tests.core;
 
 import static org.apache.felix.ipojo.tinybundles.BundleAsiPOJO.asiPOJOBundle;
-import static org.ops4j.pax.exam.CoreOptions.equinox;
 import static org.ops4j.pax.exam.CoreOptions.felix;
 import static org.ops4j.pax.exam.CoreOptions.mavenBundle;
 import static org.ops4j.pax.exam.CoreOptions.options;
 import static org.ops4j.pax.exam.CoreOptions.provision;
 import static org.ops4j.pax.exam.MavenUtils.asInProject;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.asURL;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.newBundle;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.with;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Enumeration;
 import java.util.List;
 
+import org.apache.felix.ipojo.architecture.Architecture;
 import org.apache.felix.ipojo.tests.core.component.MyComponent;
 import org.apache.felix.ipojo.tests.core.service.MyService;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.ops4j.pax.exam.Inject;
 import org.ops4j.pax.exam.Option;
 import org.ops4j.pax.exam.junit.Configuration;
 import org.ops4j.pax.exam.junit.JUnit4TestRunner;
+import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.Constants;
+import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.ServiceReference;
 import org.osgi.service.log.LogEntry;
 import org.osgi.service.log.LogReaderService;
 import org.osgi.service.log.LogService;
@@ -73,40 +76,55 @@
 
         Option[] opt =  options(
                 felix(),
-                equinox(),
+//                equinox(),
                 provision(
                         // Runtime.
-                        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.ipojo").version(asInProject()),
-                        mavenBundle().groupId( "org.ops4j.pax.swissbox" ).artifactId( "pax-swissbox-tinybundles" ).version(asInProject()),
-                        mavenBundle().groupId( "org.apache.felix" ).artifactId( "org.apache.felix.log" ).version(asInProject())
+                        mavenBundle().groupId( "org.apache.felix" ).artifactId( "org.apache.felix.log" ).version(asInProject()),
+                        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.ipojo").version(asInProject())
                         ),
                 provision(
                         newBundle()
-                            .addClass( MyService.class )
-                            .prepare()
-                           .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
-                           .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                            .build( asURL() ).toExternalForm()
+                            .add( MyService.class )
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
+                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .build()
                     ),
                provision(
                        // Component
                         newBundle()
-                            .addClass(MyComponent.class)
-                            .prepare(
-                                    with()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
-                                        .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                                        .set("ipojo-log-level", "info")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "provider-with-level-in-manifest.jar"), new File("component.xml"))).toExternalForm()
+                            .add(MyComponent.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
+                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .set("ipojo-log-level", "info")
+                            .build( asiPOJOBundle(new File(tmp, "provider-with-level-in-manifest.jar"), new File("component.xml")))
                             )
                 );
         return opt;
     }
     
     @Test
-    public void testMessages() throws InterruptedException {
+    @Ignore //TODO Why we have a classloading issue here ?
+    public void testMessages() throws InterruptedException, InvalidSyntaxException {
+        Bundle bundle = osgi.getBundle("MyComponent");
+        Assert.assertNotNull(bundle);
+        Assert.assertEquals(Bundle.ACTIVE, bundle.getState());
+        
+        ServiceReference r = ipojo.getServiceReferenceByName(Architecture.class.getName(), "org.apache.felix.ipojo.tests.core.component.MyComponent-0");
+        Assert.assertNotNull(r);
+        System.out.println(((Architecture) osgi.getServiceObject(r)).getInstanceDescription().getDescription());
+        
+        ServiceReference[] refs = context.getAllServiceReferences(null, null);
+        for (ServiceReference ref : refs) {
+            System.out.println(ref.getBundle().getBundleId() + " -> " + Arrays.asList((String[]) ref.getProperty(Constants.OBJECTCLASS)));
+        }
+        
+        
+        
+   //     Assert.assertNotNull(osgi.getServiceObject(MyService.class.getName(), null));
+        
+//        osgi.waitForService("org.apache.felix.ipojo.tests.core.service.MyService", null, 5000);
         List<String> messages = getMessages(log.getLog());
+        System.out.println(messages);
         Assert.assertTrue(messages.contains("Ready"));
         Assert.assertTrue(messages.contains("[INFO] org.apache.felix.ipojo.tests.core.component.MyComponent : Instance org.apache.felix.ipojo.tests.core.component.MyComponent-0 from factory org.apache.felix.ipojo.tests.core.component.MyComponent created"));
         Assert.assertTrue(messages.contains("[INFO] org.apache.felix.ipojo.tests.core.component.MyComponent : New factory created : org.apache.felix.ipojo.tests.core.component.MyComponent"));
diff --git a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerInfoTest.java b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerInfoTest.java
index 1a08556..75dbfab 100644
--- a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerInfoTest.java
+++ b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerInfoTest.java
@@ -8,9 +8,8 @@
 import static org.ops4j.pax.exam.CoreOptions.provision;
 import static org.ops4j.pax.exam.CoreOptions.systemProperty;
 import static org.ops4j.pax.exam.MavenUtils.asInProject;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.asURL;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.newBundle;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.with;
+import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.withBnd;
 
 import java.io.File;
 import java.util.ArrayList;
@@ -83,22 +82,18 @@
                         ),
                 provision(
                         newBundle()
-                            .addClass( MyService.class )
-                            .prepare()
-                           .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
-                           .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                            .build( asURL() ).toExternalForm()
+                            .add( MyService.class )
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
+                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .build( withBnd() )
                     ),
                provision(
                        // Component
                         newBundle()
-                            .addClass(MyComponent.class)
-                            .prepare(
-                                    with()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
-                                        .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("component.xml"))).toExternalForm()
+                            .add(MyComponent.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
+                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("component.xml")))
                             ),
                 systemProperty( "ipojo.log.level" ).value( "info" )
                 );
diff --git a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerWarningTest.java b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerWarningTest.java
index f6a5fa6..5c012a8 100644
--- a/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerWarningTest.java
+++ b/ipojo/tests/core/logger/src/test/java/org/apache/felix/ipojo/tests/core/SystemLoggerWarningTest.java
@@ -7,9 +7,8 @@
 import static org.ops4j.pax.exam.CoreOptions.provision;
 import static org.ops4j.pax.exam.CoreOptions.systemProperty;
 import static org.ops4j.pax.exam.MavenUtils.asInProject;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.asURL;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.newBundle;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.with;
+import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.withBnd;
 
 import java.io.File;
 import java.util.ArrayList;
@@ -78,27 +77,22 @@
                 provision(
                         // Runtime.
                         mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.ipojo").version(asInProject()),
-                        mavenBundle().groupId( "org.ops4j.pax.swissbox" ).artifactId( "pax-swissbox-tinybundles" ).version(asInProject()),
                         mavenBundle().groupId( "org.apache.felix" ).artifactId( "org.apache.felix.log" ).version(asInProject())
                         ),
                 provision(
                         newBundle()
-                            .addClass( MyService.class )
-                            .prepare()
+                            .add( MyService.class )
                            .set(Constants.BUNDLE_SYMBOLICNAME,"ServiceInterface")
                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                            .build( asURL() ).toExternalForm()
+                            .build( withBnd() )
                     ),
                provision(
                        // Component
                         newBundle()
-                            .addClass(MyComponent.class)
-                            .prepare(
-                                    with()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
-                                        .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("component.xml"))).toExternalForm()
+                            .add(MyComponent.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"MyComponent")
+                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.core.service")
+                            .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("component.xml")))
                             ),
                 systemProperty( "ipojo.log.level" ).value( "warning" )
                 );
diff --git a/ipojo/tests/core/service-providing-inheritance/pom.xml b/ipojo/tests/core/service-providing-inheritance/pom.xml
index 0476394..a01b289 100644
--- a/ipojo/tests/core/service-providing-inheritance/pom.xml
+++ b/ipojo/tests/core/service-providing-inheritance/pom.xml
@@ -56,58 +56,91 @@
       <version>${pom.version}</version>
     </dependency>
 
-  <!--
-    Pax Exam API:
-  -->
-  <dependency>
-    <groupId>org.ops4j.pax.exam</groupId>
-    <artifactId>pax-exam</artifactId>
-    <version>1.1.0</version>
-  </dependency>
-  <!--
-    During runtime Pax Exam will discover the OSGi container to use by
-    searching metadata available into classpath. Pax Exam comes with a
-    default container that uses [Pax Runner] for implementing the
-    container requirements:
-  -->
-  <dependency>
-    <groupId>org.ops4j.pax.exam</groupId>
-    <artifactId>pax-exam-container-default
-    </artifactId>
-    <version>1.1.0</version>
-  </dependency>
-  <!--
-    If your test code is based on JUnit you will have to have the Junit
-    support artifact:
-  -->
-  <dependency>
-    <groupId>org.ops4j.pax.exam</groupId>
-    <artifactId>pax-exam-junit</artifactId>
-    <version>1.1.0</version>
-  </dependency>
-  <dependency>
-    <groupId>junit</groupId>
-    <artifactId>junit</artifactId>
-    <version>4.5</version>
-    <type>jar</type>
-    <scope>test</scope>
-  </dependency>
-  <!--  Tinybundles -->
-  <dependency>
-    <groupId>org.ops4j.pax.swissbox</groupId>
-    <artifactId>pax-swissbox-tinybundles</artifactId>
-    <version>1.0.0</version>
-  </dependency>
+  <!-- Pax Exam API: -->
+    <dependency>
+      <groupId>org.ops4j.pax.exam</groupId>
+      <artifactId>pax-exam</artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+    <!--
+      During runtime Pax Exam will discover the OSGi container to use by
+      searching metadata available into classpath. Pax Exam comes with a
+      default container that uses [Pax Runner] for implementing the
+      container requirements:
+    -->
+    <dependency>
+      <groupId>org.ops4j.pax.exam</groupId>
+      <artifactId>pax-exam-container-default
+          </artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+    <!--
+      If your test code is based on JUnit you will have to have the Junit
+      support artifact:
+    -->
+    <dependency>
+      <groupId>org.ops4j.pax.exam</groupId>
+      <artifactId>pax-exam-junit</artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+    <dependency>
+      <groupId>junit</groupId>
+      <artifactId>junit</artifactId>
+      <version>4.5</version>
+      <type>jar</type>
+      <scope>test</scope>
+    </dependency>
+
+    <!--  TinyBundle -->
+    <dependency>
+      <groupId>org.ops4j.pax.swissbox</groupId>
+      <artifactId>pax-swissbox-tinybundles</artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+    <dependency>
+      <groupId>org.ops4j.base</groupId>
+      <artifactId>ops4j-base</artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+    <dependency>
+      <groupId>org.ops4j.pax.swissbox</groupId>
+      <artifactId>pax-swissbox-bnd</artifactId>
+      <version>1.2.0</version>
+      <scope>test</scope>
+    </dependency>
+
+    <!-- mockito -->
+    <dependency>
+      <groupId>org.mockito</groupId>
+      <artifactId>mockito-all</artifactId>
+      <version>1.7</version>
+      <scope>test</scope>
+    </dependency>
+
+    <dependency>
+      <groupId>org.apache.felix</groupId>
+      <artifactId>org.apache.felix.ipojo.tinybundles.bundleAsiPOJO
+      </artifactId>
+      <version>1.5.0-SNAPSHOT</version>
+      <scope>test</scope>
+    </dependency>
+    <dependency>
+      <groupId>org.apache.felix</groupId>
+      <artifactId>org.apache.felix.ipojo.test.helpers</artifactId>
+      <version>1.5.0-SNAPSHOT</version>
+      <scope>test</scope>
+    </dependency>
+    
   <dependency>
     <groupId>org.apache.felix</groupId>
-    <artifactId>org.apache.felix.ipojo.tinybundles.bundleAsiPOJO</artifactId>
-    <version>${pom.version}</version>
-  </dependency>
-  <dependency>
-  	<groupId>org.apache.felix</groupId>
-  	<artifactId>org.apache.felix.ipojo.annotations</artifactId>
-  	<version>1.5.0-SNAPSHOT</version>
-  	<scope>compile</scope>
+    <artifactId>org.apache.felix.ipojo.annotations</artifactId>
+    <version>1.5.0-SNAPSHOT</version>
+    <scope>compile</scope>
   </dependency>
   </dependencies>
 
diff --git a/ipojo/tests/core/service-providing-inheritance/src/test/java/org/apache/felix/ipojo/tests/inheritance/InheritanceTest.java b/ipojo/tests/core/service-providing-inheritance/src/test/java/org/apache/felix/ipojo/tests/inheritance/InheritanceTest.java
index 2e280bd..428b06e 100644
--- a/ipojo/tests/core/service-providing-inheritance/src/test/java/org/apache/felix/ipojo/tests/inheritance/InheritanceTest.java
+++ b/ipojo/tests/core/service-providing-inheritance/src/test/java/org/apache/felix/ipojo/tests/inheritance/InheritanceTest.java
@@ -7,9 +7,9 @@
 import static org.ops4j.pax.exam.CoreOptions.options;
 import static org.ops4j.pax.exam.CoreOptions.provision;
 import static org.ops4j.pax.exam.MavenUtils.asInProject;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.asURL;
+import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.withBnd;
 import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.newBundle;
-import static org.ops4j.pax.swissbox.tinybundles.core.TinyBundles.with;
+
 
 import java.io.File;
 
@@ -68,50 +68,41 @@
                 equinox(),
                 provision(
                         // Runtime.
-                        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.ipojo").version(asInProject()),
-                        mavenBundle().groupId( "org.ops4j.pax.swissbox" ).artifactId( "pax-swissbox-tinybundles" ).version(asInProject())
+                        mavenBundle().groupId("org.apache.felix").artifactId("org.apache.felix.ipojo").version(asInProject())
                         ),
                 // Bundle A
                 provision(
                         newBundle()
-                            .addClass( IA.class )
-                            .prepare()
+                            .add( IA.class )
                            .set(Constants.BUNDLE_SYMBOLICNAME,"A")
                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.a")
-                            .build( asURL() ).toExternalForm()
+                            .build( withBnd() )
                     ),
                 // Bundle B
                 provision(
                         newBundle()
-                            .addClass( IB.class )
-                            .prepare()
+                            .add( IB.class )
                            .set(Constants.BUNDLE_SYMBOLICNAME,"B")
                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.a")
                            .set(Constants.EXPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.b")
-                            .build( asURL() ).toExternalForm()
+                            .build( withBnd() )
                     ),
                // Bundle C and D : iPOJO Bundles
                provision(
                        // Component C
                         newBundle()
-                            .addClass(C.class)
-                            .prepare(
-                                    with()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"C")
-                                        .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.b," +
-                                                "org.apache.felix.ipojo.tests.inheritance.a")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("src/test/resources/provider.xml"))).toExternalForm(),
+                            .add(C.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"C")
+                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.b," +
+                                        "org.apache.felix.ipojo.tests.inheritance.a")
+                           .build( asiPOJOBundle(new File(tmp, "provider.jar"), new File("src/test/resources/provider.xml"))),
                      // Component D
                         newBundle()
-                            .addClass(D.class)
-                            .prepare(
-                                    with()
-                                        .set(Constants.BUNDLE_SYMBOLICNAME,"D")
-                                        .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.b," +
-                                                "org.apache.felix.ipojo.tests.inheritance.a")
-                                    )
-                            .build( asiPOJOBundle(new File(tmp, "cons.jar"), new File("src/test/resources/cons.xml"))).toExternalForm())
+                            .add(D.class)
+                            .set(Constants.BUNDLE_SYMBOLICNAME,"D")
+                            .set(Constants.IMPORT_PACKAGE, "org.apache.felix.ipojo.tests.inheritance.b," +
+                                    "org.apache.felix.ipojo.tests.inheritance.a")
+                            .build( asiPOJOBundle(new File(tmp, "cons.jar"), new File("src/test/resources/cons.xml"))))
                 );
         return opt;
     }
diff --git a/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/providedservice/ProvidedService.java b/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/providedservice/ProvidedService.java
index 78ad910..fa2fef8 100644
--- a/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/providedservice/ProvidedService.java
+++ b/ipojo/core/src/main/java/org/apache/felix/ipojo/handlers/providedservice/ProvidedService.java
@@ -36,6 +36,7 @@
 import org.apache.felix.ipojo.IPOJOServiceFactory;
 import org.apache.felix.ipojo.InstanceManager;
 import org.apache.felix.ipojo.util.Property;
+import org.apache.felix.ipojo.util.SecurityHelper;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.Constants;
@@ -84,7 +85,7 @@
     /**
      * At this time, it is only the java interface full name.
      */
-    private String[] m_serviceSpecification = new String[0];
+    private String[] m_serviceSpecifications = new String[0];
 
     /**
      * The service registration. is null when the service is not registered.
@@ -124,7 +125,7 @@
     public ProvidedService(ProvidedServiceHandler handler, String[] specification, int factoryPolicy, Class creationStrategyClass, Dictionary conf) {
         m_handler = handler;
 
-        m_serviceSpecification = specification;
+        m_serviceSpecifications = specification;
 
         // Add instance name, factory name and factory version is set.
         try {
@@ -191,7 +192,7 @@
                 // Thread : one service object per asking thread
                 // Consumer : one service object per consumer
                 default:
-                    List specs = Arrays.asList(m_serviceSpecification);
+                    List specs = Arrays.asList(m_serviceSpecifications);
                     m_handler.error("["
                             + m_handler.getInstanceManager().getInstanceName()
                             + "] Unknown creation policy for " + specs + " : "
@@ -312,13 +313,21 @@
     protected synchronized void registerService() {
         if (m_serviceRegistration == null) {
             // Build the service properties list
-            Properties serviceProperties = getServiceProperties();
-            m_strategy.onPublication(getInstanceManager(), m_serviceSpecification, serviceProperties);
-            m_serviceRegistration = m_handler.getInstanceManager().getContext().registerService(m_serviceSpecification, this, serviceProperties);
-            // An update may happen during the registration, re-check and apply.
-            if (m_wasUpdated) {
-                m_serviceRegistration.setProperties(getServiceProperties());
-                m_wasUpdated = false;
+            
+            BundleContext bc = m_handler.getInstanceManager().getContext();
+            // Security check
+            if (SecurityHelper.hasPermissionToRegisterServices(m_serviceSpecifications, bc)) {
+                Properties serviceProperties = getServiceProperties();
+                m_strategy.onPublication(getInstanceManager(), m_serviceSpecifications, serviceProperties);
+                m_serviceRegistration = bc.registerService(m_serviceSpecifications, this, serviceProperties);
+                // An update may happen during the registration, re-check and apply.
+                if (m_wasUpdated) {
+                    m_serviceRegistration.setProperties(getServiceProperties());
+                    m_wasUpdated = false;
+                }
+            } else {
+                throw new SecurityException("The bundle " + bc.getBundle().getBundleId() + " does not have the"
+                        + " permission to register the services " + Arrays.asList(m_serviceSpecifications));
             }      
         }
     }
@@ -429,7 +438,7 @@
      * interface).
      */
     public String[] getServiceSpecifications() {
-        return m_serviceSpecification;
+        return m_serviceSpecifications;
     }
 
     /**
@@ -641,7 +650,7 @@
          */
         public Object getService(Bundle arg0, ServiceRegistration arg1) {
             Object proxy = Proxy.newProxyInstance(getInstanceManager().getClazz().getClassLoader(),
-                    getSpecificationsWithIPOJOServiceFactory(m_serviceSpecification, m_handler.getInstanceManager().getContext()), this);
+                    getSpecificationsWithIPOJOServiceFactory(m_serviceSpecifications, m_handler.getInstanceManager().getContext()), this);
             return proxy;
         }
 